;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "use-package-treesit" "20260416.1048"
  "Automatically use tree-sitter enhanced major modes."
  '((emacs "30.0"))
  :url "https://github.com/domq/use-package-treesit.git"
  :commit "398ca0df10f2894b87c07073a446a53d943a6c18"
  :revdesc "398ca0df10f2"
  :keywords '("lisp" "tree-sitter" "treesit" "auto" "automatic" "use-package")
  :authors '(("Dominique Quatravaux" . "dominique@quatravaux.org"))
  :maintainers '(("Dominique Quatravaux" . "dominique@quatravaux.org")))
